unit DataSetResource;

// EMS Resource Module

interface

uses
  System.SysUtils, System.Classes, System.JSON,
  EMS.Services, EMS.ResourceAPI, EMS.ResourceTypes, FireDAC.Stan.Intf,
  FireDAC.Stan.Option, FireDAC.Stan.Param, FireDAC.Stan.Error, FireDAC.DatS,
  FireDAC.Phys.Intf, FireDAC.DApt.Intf, FireDAC.Stan.Async, FireDAC.DApt,
  FireDAC.UI.Intf, FireDAC.Stan.Def, FireDAC.Stan.Pool, FireDAC.Phys,
  FireDAC.Phys.IB, FireDAC.Phys.IBDef, FireDAC.ConsoleUI.Wait, Data.DB,
  FireDAC.Stan.StorageJSON, FireDAC.Comp.Client, FireDAC.Comp.DataSet;

type
  [ResourceName('datasets')]
  TDatasetResource1 = class(TDataModule)
    qryBook: TFDQuery;
    conBookRental: TFDConnection;
    scmAdtBookRental: TFDSchemaAdapter;
    FDStanStorageJSONLink1: TFDStanStorageJSONLink;
    qryUser: TFDQuery;
    qryBookBOOK_SEQ: TFDAutoIncField;
    qryBookBOOK_TITLE: TWideStringField;
    qryBookBOOK_AUTHOR: TWideStringField;
    qryBookBOOK_ISBN: TStringField;
    qryBookBOOK_PRICE: TIntegerField;
    qryBookBOOK_LINK: TWideStringField;
  published
    procedure Get(const AContext: TEndpointContext; const ARequest: TEndpointRequest; const AResponse: TEndpointResponse);
    procedure Post(const AContext: TEndpointContext; const ARequest: TEndpointRequest; const AResponse: TEndpointResponse);
    [ResourceSuffix('/books/')]
    procedure GetItemBooks(const AContext: TEndpointContext; const ARequest: TEndpointRequest; const AResponse: TEndpointResponse);
    [ResourceSuffix('/books/')]
    procedure PostBooks(const AContext: TEndpointContext; const ARequest: TEndpointRequest; const AResponse: TEndpointResponse);
  end;

implementation

{%CLASSGROUP 'System.Classes.TPersistent'}

{$R *.dfm}

procedure TDatasetResource1.Get(const AContext: TEndpointContext; const ARequest: TEndpointRequest; const AResponse: TEndpointResponse);
var
  Stream: TMemoryStream;
begin
  qryBook.Open;
  Stream := TMemoryStream.Create;
  try

    scmAdtBookRental.SaveToStream(Stream, TFDStorageFormat.sfJSON);

    AResponse.Body.SetStream(Stream, 'application/json', True);
  except
    Stream.DisposeOf;
  end;
end;

procedure TDatasetResource1.GetItemBooks(const AContext: TEndpointContext;
  const ARequest: TEndpointRequest; const AResponse: TEndpointResponse);
var
  Stream: TMemoryStream;
begin
  qryBook.Open;
  Stream := TMemoryStream.Create;
  try

    qryBook.SaveToStream(Stream, TFDStorageFormat.sfJSON);
//    scmAdtBookRental.SaveToStream(Stream, TFDStorageFormat.sfJSON);

    AResponse.Body.SetStream(Stream, 'application/json', True);
  except
    Stream.DisposeOf;
  end;
end;

procedure TDatasetResource1.Post(const AContext: TEndpointContext;
  const ARequest: TEndpointRequest; const AResponse: TEndpointResponse);
var
  Stream: TStream;
begin
  if not SameText(ARequest.Body.ContentType, 'application/json') then
    AResponse.RaiseBadRequest('content type');
  if not ARequest.Body.TryGetStream(Stream) then
    AResponse.RaiseBadRequest('Invailed stream');

  Stream.Position := 0;
  scmAdtBookRental.LoadFromStream(Stream, TFDStorageFormat.sfJSON);
  scmAdtBookRental.ApplyUpdates;
end;

procedure TDatasetResource1.PostBooks(const AContext: TEndpointContext;
  const ARequest: TEndpointRequest; const AResponse: TEndpointResponse);
var
  Stream: TStream;
begin
  if not SameText(ARequest.Body.ContentType, 'application/json') then
    AResponse.RaiseBadRequest('content type');
  if not ARequest.Body.TryGetStream(Stream) then
    AResponse.RaiseBadRequest('Invailed stream');

  Stream.Position := 0;
//  qryBook.Open;
  qryBook.LoadFromStream(Stream, TFDStorageFormat.sfJSON);
  qryBook.ApplyUpdates;
end;

procedure Register;
begin
  RegisterResource(TypeInfo(TDatasetResource1));
end;

initialization
  Register;
end.


